#version 330

float pLightAtt;

in vec3 Position;
in vec3 Normal;
in vec3 TexCoord;
in vec3 EyeDirection_cameraspace;

uniform sampler2D textureMap;
uniform mat4 ViewMatrix;
uniform int numLights;
uniform int perVertex;

struct LightInfo
{
	vec4 Position; // Light position in eye coords.
	vec4 Target;
	vec2 Intensity; // A,D intensity
	float Multiplier;
	int Mode;	//0 omni 1 directional
	float lightConstAtt;
	float lightLinAtt;
	float lightQuadAtt;
	
};
uniform LightInfo Light[20];

struct MaterialInfo
{
	vec3 Ka; // Ambient reflectivity
	vec3 Kd; // Diffuse reflectivity
};
uniform MaterialInfo Material[20];

layout( location = 0 ) out vec4 FragColor;

void phongModel( int index, vec3 pos, vec3 norm, out vec3 amb, out vec3 diff)
{
	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec4 LightPosition_cameraspace = ViewMatrix * Light[index].Position;
	vec3 LightDirection_cameraspace = LightPosition_cameraspace.xyz + EyeDirection_cameraspace;

	norm = normalize( norm );
	
	// Distance to the light
	float dist = length( Light[index].Position.xyz - Position	)/1000;	
	if(Light[index].Mode==1)
	{
		LightDirection_cameraspace = Light[index].Position.xyz- Light[index].Target.xyz;
		dist = 1;	
	}
	
	vec3 s = normalize(LightDirection_cameraspace);
	
	pLightAtt =1 / (Light[index].lightConstAtt + Light[index].lightLinAtt*dist + Light[index].lightQuadAtt*dist*dist);
	// Direction of the light (from the fragment to the light)
	float cosTheta = dot(norm,s);
	float fFactor = clamp( cosTheta, 0.f , 1.f );
	
	diff= pLightAtt * Light[index].Multiplier * Light[index].Intensity.y  * Material[index].Kd * fFactor;	
	amb = Light[index].Multiplier * Light[index].Intensity.x * Material[index].Ka;
}

void main()
{
	vec3 amb, diff, spec;
	vec4 texCol=texCoord;
	if(perVertex=0)
		texture( textureMap, vec2(texCoord.x,texCoord.y);	
	
	for(int i=0;i<numLights;i++)
	{
		phongModel(i,Position, Normal, amb, diff);
		FragColor += vec4(amb+diff, 1.0)*texCol;
	}
	
	FragColor=clamp(FragColor,0.0,1.0);
}