#version 330

layout (location = 0) in vec3 VertexPosition;
layout (location = 1) in vec2 VertexTexCoord;
layout (location = 2) in vec3 VertexNormal;

out vec3 Position;
out vec3 Normal;
out vec2 TexCoord;
out vec3 EyeDirection_cameraspace;

uniform mat4 ModelViewMatrix;
uniform mat3 NormalMatrix;
uniform mat4 ViewMatrix;
uniform mat4 MVP;
uniform mat4 M;

void main()
{
	// Output position of the vertex, in clip space : MVP * position
	gl_Position =  MVP * vec4(VertexPosition,1);
	
	// Position of the vertex, in worldspace : M * position
	vec4 esPos=M * vec4(VertexPosition,1);
	Position = (vec3 (esPos.xyz)) / esPos.w;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 vertexPosition_cameraspace = ( ModelViewMatrix * vec4(VertexPosition,1)).xyz;
	EyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;
	
	// Normal of the the vertex, in camera space
	Normal = NormalMatrix * vec4(VertexNormal,0).xyz;
	
	// UV of the vertex. No special space for this one.
	TexCoord = VertexTexCoord;
}