#version 330 

in VertexData
{
	vec3 position;
    vec3 texCoord;
    vec3 normal;
	vec3 weight;
	vec3 color;
} VertexIn;

uniform mat4 ViewMatrix;
uniform sampler2D lineTextureMap;
uniform sampler2D meshTextureMap;
uniform int numLights;
uniform int lineMode;
uniform int perVertex;

struct LightInfo
{
	vec4 Position; // Light position in eye coords.
	vec4 Target;
	vec2 Intensity; // A,D intensity
	float Multiplier;
	int Mode;	//0 omni 1 directional
	float lightConstAtt;
	float lightLinAtt;
	float lightQuadAtt;
	
};
uniform LightInfo Light[20];

struct MaterialInfo
{
	vec3 Ka; // Ambient reflectivity
	vec3 Kd; // Diffuse reflectivity
};
uniform MaterialInfo Material[20];

layout( location = 0 ) out vec4 FragColor;

float edgeFactor()
{
        vec3 d = fwidth(VertexIn.weight);
        vec3 a3 = smoothstep(vec3(0.0), d*0.95, VertexIn.weight);
        return min(min(a3.x, a3.y), a3.z);
}

vec3 phongModel( int index, vec3 pos, vec3 norm)
{
	vec4 lPosEye = ViewMatrix * Light[index].Position;
	vec3 s = normalize(lPosEye.xyz - pos);
	
	// Distance to the light
	float dist = length(lPosEye.xyz - pos)/100000;	
	if(Light[index].Mode==1)
	{
		s = normalize(Light[index].Position.xyz- Light[index].Target.xyz);
		dist = 1;	
	}
	
	
	float sDotN = max( dot(s,norm), 0.0 );
	sDotN = clamp(sDotN ,0.0 ,1.0);
	float pLightAtt =1 / (Light[index].lightConstAtt + Light[index].lightLinAtt*dist + Light[index].lightQuadAtt*dist*dist);
	vec3 ambient = Light[index].Multiplier * Light[index].Intensity.x * Material[index].Ka;				
	vec3 diffuse = pLightAtt * Light[index].Multiplier * Light[index].Intensity.y * Material[index].Kd * sDotN;
	
	return ambient + diffuse;
}

void main()
{
	vec4 meshCol=vec4(VertexIn.texCoord,1.0);
	if(perVertex==0)
		meshCol=texture( meshTextureMap, vec2(VertexIn.texCoord.x,VertexIn.texCoord.y));

	if(lineMode==0)
	{
		FragColor = meshCol;
	}
	else
	{
		vec4 col;
		for(int i=0;i<numLights;i++)
		{
			col += vec4(phongModel(i,VertexIn.position, VertexIn.normal), 1.0)*meshCol;
		}
		if(lineMode==1)
		{
			FragColor = col;
		}
		else
		{
			vec4 lineCol=texture(lineTextureMap, vec2(0.0,0.0));
			float edge=edgeFactor();
			
			if(lineMode==2)
			{	
				FragColor = mix( lineCol, col, edge );
			}
			else if(lineMode==3)
			{	
				if(edge>0.9)
					discard;
			
				FragColor = lineCol;
			}
		}
	}
}
	